from apistar import Include, Route, http
from apistar.interfaces import Router
from apistar.frameworks.wsgi import WSGIApp as App
from apistar.backends import sqlalchemy_backend
from apistar.backends.sqlalchemy_backend import Session
from apistar.handlers import docs_urls, static_urls

from .models import Base, NewsArticle, NewsSource
from .renders import JSONRenderer
from .schema import NewsSourceSchema

news_source_schema = NewsSourceSchema()


def get_sources(session: Session):
    sources = session.query(NewsSource).all()
    return news_source_schema.dump(sources, many=True).data


def add_source(session: Session, request_data: http.RequestData, router: Router):
    news_source_data, errors = news_source_schema.load(request_data)
    if errors:
        msg = {"message": "400 Bad Request", "error": errors}
        return http.Response(msg, status=400)

    news_source = NewsSource(**news_source_data)
    session.add(news_source)
    session.flush()

    headers = {"Location": router.reverse_url('get_source', {"id": news_source.id})}

    return http.Response(news_source_schema.dump(news_source).data, status=201, headers=headers)


def delete_source(session: Session, id: int):
    """Delete a single News Sources from the collection by id"""
    news_source = session.query(NewsSource).filter_by(id=id).one_or_none()
    if news_source is None:
        msg = {"message": "404 Not Found"}
        return http.Response(msg, status=404)

    msg = {"message": "200 OK"}
    return http.Response(msg, status=200)


def get_source(session: Session, id: int):
    news_source = session.query(NewsSource).filter_by(id=id).one_or_none()
    if news_source is None:
        msg = {"message": "404 Not Found"}
        return http.Response(msg, status=404)

    return http.Response(news_source_schema.dump(news_source).data, status=200)


def get_articles(session: Session):
    articles = session.query(NewsArticle).all()
    return [article.to_dict() for article in articles]


def add_article(session: Session):
    pass


def delete_article(session: Session):
    pass


routes = [
    Route('/sources', 'GET', get_sources),
    Route('/sources', 'POST', add_source),
    Route('/sources/{id}', 'GET', get_source),
    Route('/sources/{id}', 'DELETE', delete_source),
    Route('/articles', 'GET', get_articles),
    Include('/docs', docs_urls),
    Include('/static', static_urls)
]

_settings = {
    'DATABASE': {
        'URL': 'postgresql://apistar:local@localhost/news',
        'METADATA': Base.metadata
    },
    'RENDERERS': [JSONRenderer()]
}

routes = routes

commands = sqlalchemy_backend.commands

components = sqlalchemy_backend.components


def application_factory(settings={}):
    """Returns an instance of Cookie API"""
    app_settings = {**_settings, **settings}
    return App(settings=app_settings,
               commands=commands,
               components=components,
               routes=routes)
